import requests
import json
import urllib3
import logging
import argparse
import os
import re
import csv
import time
import sys

dir_path = os.path.dirname(os.path.realpath(__file__))
parent_dir_path = os.path.abspath(os.path.join(dir_path, os.pardir))
sys.path.insert(0, parent_dir_path)

from clients import DynamoDbClient

from botocore.exceptions import ClientError

urllib3.disable_warnings()
logger = logging.getLogger("filter-global-roles")
logger.setLevel(logging.INFO)
formatter = logging.Formatter("%(asctime)s %(levelname)s - %(message)s")
handler_console = logging.StreamHandler()
handler_console.setFormatter(formatter)
logger.addHandler(handler_console)



def main():
    environment_choices = ['localdev', 'prodsyd']

    parser = argparse.ArgumentParser(description="Arg Parser for filtering roles")
    parser.add_argument('-e', '--environment', required=True,
                        choices=environment_choices)
    parser.add_argument("-a", "--apply", help="Apply the changes. Plan is done by default", action="store_true")

    args = parser.parse_known_args()[0]

    env = args.environment
    apply = args.apply

    dynamo_client = DynamoDbClient(env)
    dynamo_client.setClientEnvironment()

    logger.info("Running filter for global rbac entities")
    table_name = '{}_access_roles'.format(get_hub(env))

    scan_res = dynamo_client.scan(table_name)
    items = scan_res['Items']
    logger.info('%d items in %s' % (len(items), table_name))

    deleted_items = []

    # Before delete
    export_items(items, 'before')

    deleted_count = 0
    for item in items:
        if not is_global_rbac_entity(item):
            try:
                if apply:
                    key = {"urn": {"S": "{}".format(item['urn']['S'])},
                           "child_urn": {"S": "{}".format(item['child_urn']['S'])}}
                    logger.info("Deleting item with key %s" % key)
                    dynamo_client.delete_item(table_name, key)

                deleted_items.append(item)
                deleted_count += 1
            except ClientError as ex:
                logger.error("Delete item failed, error: %s)" % ex.response)


    plan_delete_name = 'deleted' if apply else 'planned'
    # Exporting items deleted/planned
    export_items(deleted_items, '{}_items'.format(plan_delete_name))

    logger.info("%s %d items from dynamo table" % (plan_delete_name, deleted_count))

    if apply:
        # Performing Scan to see table after
        scan_res_after = dynamo_client.scan(table_name)
        items_after = scan_res_after['Items']
        export_items(items_after, 'after')



"""
    Given a list of dictionary items, export into a csv
"""
def export_items(items, csv_name):
    csv_columns = ['urn', 'child_urn', 'scoped']
    with open('{}-{}.csv'.format(csv_name, time.time()), 'w') as f:
        writer = csv.writer(f, delimiter=',')
        writer.writerow(['urn', 'child_urn', 'scoped'])
        for item in items:
            if 'scoped' in item.keys():
                writer.writerow([item['urn']['S'],
                                 item['child_urn']['S'],
                                 item['scoped']['S']])
            else:
                writer.writerow([item['urn']['S'],
                                 item['child_urn']['S']])

"""
    Determine if a dictionary item is a global RBAC entity
    If an item has an ORGKEy in the urn or child urn it s not a global definition. 
    We are also excluding any cache dynamo entries
"""
def is_global_rbac_entity(item):
    org_role_pattern = re.compile('^(psc:role:.+:).+')

    urn = item['urn']['S']
    child_urn = item['child_urn']['S']

    # match variable contains a Match object.
    match_urn = re.search(org_role_pattern, urn)
    match_child_urn = re.search(org_role_pattern, child_urn)

    if match_urn or match_child_urn:
        print('Non global entity found urn: {}, child_urn: {}'.format(urn, child_urn))
        return False

    if 'cache' in child_urn:
        print('Non global entity found urn: {}, child_urn: {}'.format(urn, child_urn))
        return False

    if 'scoped' in item.keys():
        if 'cache' in item['scoped']['S']:
            print('Non global entity found urn: {}, child_urn: {}'.format(urn, child_urn))
            return False

    return True

"""
    Given the environment, return the hub name
"""
def get_hub(environment):
    hub = None
    localdev_hub = ['localdev']
    prod_ap2_hub = ['prodsyd']

    if environment in localdev_hub:
        hub = 'localdev'
    elif environment in prod_ap2_hub:
        hub = 'hub-prod-ap2'

    if hub == None:
        raise Exception("Could not get Hub configuration for environment {}".format(environment))
    logger.info("Hub Environment %s " % hub)
    return hub

if __name__ == "__main__":
    main()